(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var _ = Package.underscore._;
var check = Package.check.check;
var Match = Package.check.Match;
var loglevel = Package['practicalmeteor:loglevel'].loglevel;
var MeteorFilesHelpers = Package['sanjo:meteor-files-helpers'].MeteorFilesHelpers;
var LongRunningChildProcess = Package['sanjo:long-running-child-process'].LongRunningChildProcess;

/* Package-scope variables */
var Karma, KarmaInternals, log;

(function () {

/////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                 //
// packages/sanjo:karma/main.js                                                                    //
//                                                                                                 //
/////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                   //
var path = Npm.require('path')                                                                     // 1
var fs = Npm.require('fs-extra')                                                                   // 2
                                                                                                   // 3
log = loglevel.createPackageLogger('[sanjo:karma]', process.env.KARMA_LOG_LEVEL || 'info')         // 4
                                                                                                   // 5
Karma = {                                                                                          // 6
  start: function (id, options) {                                                                  // 7
    options = options || {}                                                                        // 8
    log.debug('Karma.start', id)                                                                   // 9
                                                                                                   // 10
    Karma.setConfig(id, options)                                                                   // 11
                                                                                                   // 12
    return KarmaInternals.startKarmaServer(id, options)                                            // 13
  },                                                                                               // 14
                                                                                                   // 15
  setConfig: function (id, options) {                                                              // 16
    log.debug('Karma.setConfig', id)                                                               // 17
                                                                                                   // 18
    var oldConfig = KarmaInternals.readKarmaConfig(id)                                             // 19
    var newConfig = KarmaInternals.generateKarmaConfig(options)                                    // 20
                                                                                                   // 21
    if (!oldConfig || newConfig !== oldConfig) {                                                   // 22
      log.debug('New config is different from the old one.')                                       // 23
      log.debug(oldConfig)                                                                         // 24
      log.debug(newConfig)                                                                         // 25
      KarmaInternals.writeKarmaConfig(id, newConfig)                                               // 26
                                                                                                   // 27
      // Restart running Karma server when config has changed                                      // 28
      var karmaChild = KarmaInternals.getKarmaChild(id)                                            // 29
      if (karmaChild.isRunning()) {                                                                // 30
        log.debug('Restarting Karma server to reload config.')                                     // 31
        karmaChild.kill()                                                                          // 32
        KarmaInternals.startKarmaServer(id)                                                        // 33
      }                                                                                            // 34
    } else {                                                                                       // 35
      log.debug('New config is exactly the same as the old one.')                                  // 36
    }                                                                                              // 37
  }                                                                                                // 38
}                                                                                                  // 39
                                                                                                   // 40
KarmaInternals = {                                                                                 // 41
  karmaChilds: {},                                                                                 // 42
                                                                                                   // 43
  getKarmaChild: function (id) {                                                                   // 44
    var karmaChild = KarmaInternals.karmaChilds[id]                                                // 45
    if (!karmaChild) {                                                                             // 46
      karmaChild = new sanjo.LongRunningChildProcess(id)                                           // 47
      KarmaInternals.setKarmaChild(id, karmaChild)                                                 // 48
    }                                                                                              // 49
                                                                                                   // 50
    return karmaChild                                                                              // 51
  },                                                                                               // 52
                                                                                                   // 53
  setKarmaChild: function (id, karmaChild) {                                                       // 54
    KarmaInternals.karmaChilds[id] = karmaChild                                                    // 55
  },                                                                                               // 56
                                                                                                   // 57
  startKarmaServer: function (id) {                                                                // 58
    log.debug('KarmaInternals.startKarmaServer(' + id + ')')                                       // 59
    var karmaChild = KarmaInternals.getKarmaChild(id)                                              // 60
    var configPath = KarmaInternals.getConfigPath(id)                                              // 61
    var karmaPath = KarmaInternals.getKarmaPath();                                                 // 62
    fs.chmodSync(karmaPath, parseInt('544', 8));                                                   // 63
    var spawnOptions = {                                                                           // 64
      command: process.execPath,                                                                   // 65
      args: [karmaPath, 'start', configPath]                                                       // 66
    }                                                                                              // 67
    // It will only spawn when the process is not already running                                  // 68
    karmaChild.spawn(spawnOptions)                                                                 // 69
                                                                                                   // 70
    return karmaChild                                                                              // 71
  },                                                                                               // 72
                                                                                                   // 73
  writeKarmaConfig: function (id, config) {                                                        // 74
    var configPath = KarmaInternals.getConfigPath(id)                                              // 75
    fs.outputFileSync(configPath, config)                                                          // 76
                                                                                                   // 77
    return configPath                                                                              // 78
  },                                                                                               // 79
                                                                                                   // 80
  generateKarmaConfig: function (options) {                                                        // 81
    return 'module.exports = function(config) {\n' +                                               // 82
      '  config.set(' +                                                                            // 83
      JSON.stringify(options, null, 2) +                                                           // 84
      ');\n};'                                                                                     // 85
  },                                                                                               // 86
                                                                                                   // 87
  readKarmaConfig: function (id) {                                                                 // 88
    var configPath = KarmaInternals.getConfigPath(id)                                              // 89
    try {                                                                                          // 90
      return fs.readFileSync(configPath, {encoding: 'utf8'})                                       // 91
    } catch (error) {                                                                              // 92
      return null;                                                                                 // 93
    }                                                                                              // 94
  },                                                                                               // 95
                                                                                                   // 96
  getConfigPath: function (id) {                                                                   // 97
    return path.join(                                                                              // 98
      MeteorFilesHelpers.getAppPath(),                                                             // 99
      '.meteor/local/karma/' + id + '.config.js'                                                   // 100
    )                                                                                              // 101
  },                                                                                               // 102
                                                                                                   // 103
  getKarmaPath: function () {                                                                      // 104
    return path.join(MeteorFilesHelpers.getNodeModulePath('sanjo:karma', 'karma'), 'bin', 'karma') // 105
  }                                                                                                // 106
}                                                                                                  // 107
                                                                                                   // 108
/////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['sanjo:karma'] = {
  Karma: Karma,
  KarmaInternals: KarmaInternals
};

})();

//# sourceMappingURL=sanjo_karma.js.map
