(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var _ = Package.underscore._;
var HTTP = Package.http.HTTP;
var HTTPInternals = Package.http.HTTPInternals;
var JsonRoutes = Package['simple:json-routes'].JsonRoutes;
var LongRunningChildProcess = Package['sanjo:long-running-child-process'].LongRunningChildProcess;

/* Package-scope variables */
var DEBUG;

(function () {

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                    //
// packages/xolvio:cucumber/src/hub-server.js                                                                         //
//                                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                      //
/*jshint -W030, -W020 */                                                                                              // 1
                                                                                                                      // 2
DEBUG = !!process.env.VELOCITY_DEBUG;                                                                                 // 3
                                                                                                                      // 4
(function () {                                                                                                        // 5
                                                                                                                      // 6
  'use strict';                                                                                                       // 7
                                                                                                                      // 8
  var path = Npm.require('path');                                                                                     // 9
                                                                                                                      // 10
  var FRAMEWORK_NAME  = 'cucumber',                                                                                   // 11
      FRAMEWORK_REGEX = '^tests/' + FRAMEWORK_NAME + '/((?!node_modules/).)+\\.(feature|js|coffee|litcoffee|coffee\\.md)$',
      SAMPLE_TESTS    = [{                                                                                            // 13
        contents: Assets.getText(path.join('src', 'sample-tests', 'feature.feature')),                                // 14
        path: path.join(FRAMEWORK_NAME, 'features', 'sample.feature')                                                 // 15
      }, {                                                                                                            // 16
        contents: Assets.getText(path.join('src', 'sample-tests', 'steps.js')),                                       // 17
        path: path.join(FRAMEWORK_NAME, 'features', 'step_definitions', 'sample_steps.js')                            // 18
      }, {                                                                                                            // 19
        contents: Assets.getText(path.join('src', 'sample-tests', 'fixture.js')),                                     // 20
        path: path.join(FRAMEWORK_NAME, 'fixtures', 'my_fixture.js')                                                  // 21
      }, {                                                                                                            // 22
        contents: Assets.getText(path.join('src', 'sample-tests', 'package.json')),                                   // 23
        path: path.join(FRAMEWORK_NAME, 'package.json')                                                               // 24
      }];                                                                                                             // 25
                                                                                                                      // 26
  if (!!process.env.INSTALL_DEPENDENCIES) {                                                                           // 27
    _installDependencies();                                                                                           // 28
  }                                                                                                                   // 29
                                                                                                                      // 30
  if (process.env.NODE_ENV !== 'development' || process.env.IS_MIRROR ||                                              // 31
    process.env[FRAMEWORK_NAME.toUpperCase()] === '0' || process.env.VELOCITY === '0') {                              // 32
    return;                                                                                                           // 33
  }                                                                                                                   // 34
                                                                                                                      // 35
  DEBUG && console.log('[xolvio:cucumber] Cucumber hub is loading');                                                  // 36
                                                                                                                      // 37
  if (Velocity && Velocity.registerTestingFramework) {                                                                // 38
    Velocity.registerTestingFramework(FRAMEWORK_NAME, {                                                               // 39
      regex: FRAMEWORK_REGEX,                                                                                         // 40
      sampleTestGenerator: function () { return SAMPLE_TESTS; }                                                       // 41
    });                                                                                                               // 42
  }                                                                                                                   // 43
                                                                                                                      // 44
  Meteor.methods({                                                                                                    // 45
    'cucumber/run': function () {                                                                                     // 46
      this.unblock();                                                                                                 // 47
      var mirror = DDP.connect(VelocityMirrors.findOne({framework: 'cucumber'}).host)                                 // 48
      mirror.call('runAll', function () {                                                                             // 49
        mirror.disconnect();                                                                                          // 50
      });                                                                                                             // 51
    }                                                                                                                 // 52
  });                                                                                                                 // 53
                                                                                                                      // 54
  Meteor.startup(function () {                                                                                        // 55
    Meteor.call('velocity/mirrors/request', {                                                                         // 56
      framework: FRAMEWORK_NAME,                                                                                      // 57
      args: ['--raw-logs'],                                                                                           // 58
      testsPath: path.join(FRAMEWORK_NAME, 'fixtures'),                                                               // 59
      nodes: process.env.CUCUMBER_NODES ? parseInt(process.env.CUCUMBER_NODES) : 1                                    // 60
    });                                                                                                               // 61
                                                                                                                      // 62
    if (process.env.CUCUMBER_TAIL) {                                                                                  // 63
      _tailCucumberLogs();                                                                                            // 64
    }                                                                                                                 // 65
                                                                                                                      // 66
                                                                                                                      // 67
    var initOnce = _.once(Meteor.bindEnvironment(_init));                                                             // 68
    VelocityMirrors.find({framework: FRAMEWORK_NAME, state: 'ready'}).observe({                                       // 69
      added: initOnce,                                                                                                // 70
      changed: initOnce                                                                                               // 71
    });                                                                                                               // 72
  });                                                                                                                 // 73
                                                                                                                      // 74
  function _init () {                                                                                                 // 75
                                                                                                                      // 76
    if (!!process.env.CUCUMBER_NODES) {                                                                               // 77
                                                                                                                      // 78
      var debouncedRun = _.debounce(Meteor.bindEnvironment(_run), 600);                                               // 79
      VelocityTestFiles.find({targetFramework: FRAMEWORK_NAME}).observeChanges({                                      // 80
        added: debouncedRun,                                                                                          // 81
        removed: debouncedRun,                                                                                        // 82
        changed: debouncedRun                                                                                         // 83
      });                                                                                                             // 84
      process.on('SIGUSR2', Meteor.bindEnvironment(debouncedRun));                                                    // 85
      process.on('message', Meteor.bindEnvironment(function (message) {                                               // 86
        if (message.refresh && message.refresh === 'client') {                                                        // 87
          debouncedRun();                                                                                             // 88
        }                                                                                                             // 89
      }));                                                                                                            // 90
                                                                                                                      // 91
    }                                                                                                                 // 92
  }                                                                                                                   // 93
                                                                                                                      // 94
  function _tailCucumberLogs () {                                                                                     // 95
                                                                                                                      // 96
    var cucumberLogFilePath = path.join(process.env.PWD, '.meteor', 'local', 'log', 'cucumber.log')                   // 97
    var Tail = Npm.require('tail-forever');                                                                           // 98
    var tail = new Tail(cucumberLogFilePath, {});                                                                     // 99
                                                                                                                      // 100
    tail.on('line', function (data) {                                                                                 // 101
      console.log('[cucumber.log]', data);                                                                            // 102
      if (data.indexOf('Exiting cucumber') !== -1) {                                                                  // 103
        tail.unwatch();                                                                                               // 104
      }                                                                                                               // 105
    });                                                                                                               // 106
                                                                                                                      // 107
    tail.on('error', function (error) {                                                                               // 108
      console.error('[cucumber.log]', error);                                                                         // 109
    });                                                                                                               // 110
                                                                                                                      // 111
  }                                                                                                                   // 112
                                                                                                                      // 113
  function _run (id, changes) {                                                                                       // 114
                                                                                                                      // 115
    if (changes && changes.status) {                                                                                  // 116
      // one of the files has changed status. good time to check if all files are done                                // 117
      if (VelocityTestFiles.find({targetFramework: FRAMEWORK_NAME, status: 'DONE'}).count ===                         // 118
        VelocityTestFiles.find({targetFramework: FRAMEWORK_NAME}).count) {                                            // 119
        Meteor.call('velocity/reports/completed', {framework: FRAMEWORK_NAME});                                       // 120
      }                                                                                                               // 121
      return;                                                                                                         // 122
    }                                                                                                                 // 123
                                                                                                                      // 124
    Meteor.call('velocity/reports/reset', {framework: FRAMEWORK_NAME});                                               // 125
                                                                                                                      // 126
    // set all file statuses so mirrors can start working on files                                                    // 127
    VelocityTestFiles.update(                                                                                         // 128
      {targetFramework: FRAMEWORK_NAME, relativePath: /\.feature$/},                                                  // 129
      {$set: {status: 'TODO'}},                                                                                       // 130
      {multi: true}                                                                                                   // 131
    );                                                                                                                // 132
                                                                                                                      // 133
  }                                                                                                                   // 134
                                                                                                                      // 135
  function _installDependencies () {                                                                                  // 136
    DEBUG && console.log('[xolvio:cucumber] Attempting to install chimp dependencies');                               // 137
    if (DEBUG) {                                                                                                      // 138
      process.env['chimp.debug'] = true;                                                                              // 139
    }                                                                                                                 // 140
    Meteor.wrapAsync(Npm.require('chimp').install)();                                                                 // 141
  }                                                                                                                   // 142
                                                                                                                      // 143
                                                                                                                      // 144
})();                                                                                                                 // 145
                                                                                                                      // 146
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);






(function () {

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                    //
// packages/xolvio:cucumber/src/mirror-server.js                                                                      //
//                                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                      //
/*jshint -W030, -W020 */                                                                                              // 1
                                                                                                                      // 2
DEBUG = !!process.env.VELOCITY_DEBUG;                                                                                 // 3
                                                                                                                      // 4
(function () {                                                                                                        // 5
                                                                                                                      // 6
  'use strict';                                                                                                       // 7
                                                                                                                      // 8
  var path     = Npm.require('path'),                                                                                 // 9
      fs       = Npm.require('fs-extra'),                                                                             // 10
      freeport = Npm.require('freeport');                                                                             // 11
                                                                                                                      // 12
  // this library extends the string prototype                                                                        // 13
  Npm.require('colors');                                                                                              // 14
                                                                                                                      // 15
  var FRAMEWORK_NAME = 'cucumber';                                                                                    // 16
  var WATCH_TAG = '@dev';                                                                                             // 17
  var BINARY = process.env.CHIMP_PATH || Npm.require('chimp').bin;                                                    // 18
  if (                                                                                                                // 19
    process.env.NODE_ENV !== 'development' || !process.env.IS_MIRROR ||                                               // 20
    process.env[FRAMEWORK_NAME.toUpperCase()] === '0' ||                                                              // 21
    process.env.VELOCITY === '0' ||                                                                                   // 22
    process.env.FRAMEWORK.indexOf(FRAMEWORK_NAME) !== 0                                                               // 23
  ) {                                                                                                                 // 24
    return;                                                                                                           // 25
  }                                                                                                                   // 26
                                                                                                                      // 27
  DEBUG && console.log('[xolvio:cucumber] Mirror server is initializing');                                            // 28
                                                                                                                      // 29
  var _velocityConnection,                                                                                            // 30
      _velocityTestFiles,                                                                                             // 31
      _chimpProc,                                                                                                     // 32
      _chimpServerPort,                                                                                               // 33
      _runningParallelTest;                                                                                           // 34
                                                                                                                      // 35
  Meteor.startup(_startChimp);                                                                                        // 36
                                                                                                                      // 37
                                                                                                                      // 38
                                                                                                                      // 39
  Meteor.methods({                                                                                                    // 40
    handshake: function () {                                                                                          // 41
      this.unblock();                                                                                                 // 42
      DEBUG && console.log('[xolvio:cucumber] Received handshake from Chimp.');                                       // 43
      _init();                                                                                                        // 44
    },                                                                                                                // 45
    runAll: function () {                                                                                             // 46
      this.unblock();                                                                                                 // 47
      DEBUG && console.log('[xolvio:cucumber] Running all specs.');                                                   // 48
      _runSingleExecutionMode(true);                                                                                  // 49
    }                                                                                                                 // 50
  });                                                                                                                 // 51
                                                                                                                      // 52
  function _findAndRun () {                                                                                           // 53
    DEBUG && console.log('[xolvio:cucumber] Find and run triggered', arguments);                                      // 54
                                                                                                                      // 55
    var findAndRun = function () {                                                                                    // 56
      var feature = _velocityConnection.call('velocity/returnTODOTestAndMarkItAsDOING', {framework: FRAMEWORK_NAME}); // 57
      if (feature) {                                                                                                  // 58
        _runningParallelTest = true;                                                                                  // 59
        _run(feature, findAndRun);                                                                                    // 60
      }                                                                                                               // 61
                                                                                                                      // 62
    };                                                                                                                // 63
                                                                                                                      // 64
    if (!!process.env.CUCUMBER_NODES) {                                                                               // 65
      DEBUG && console.log('[xolvio:cucumber] Running in split-features mode');                                       // 66
      !_runningParallelTest && findAndRun();                                                                          // 67
    } else {                                                                                                          // 68
      DEBUG && console.log('[xolvio:cucumber] Running in batch-features mode.');                                      // 69
      _run();                                                                                                         // 70
    }                                                                                                                 // 71
                                                                                                                      // 72
  }                                                                                                                   // 73
                                                                                                                      // 74
  function _init () {                                                                                                 // 75
                                                                                                                      // 76
    DEBUG && console.log('[xolvio:cucumber] Connecting to hub');                                                      // 77
                                                                                                                      // 78
    _velocityConnection = DDP.connect(process.env.PARENT_URL);                                                        // 79
    _velocityConnection.subscribe('VelocityTestFiles');                                                               // 80
                                                                                                                      // 81
    _velocityTestFiles = new Mongo.Collection('velocityTestFiles', {                                                  // 82
      connection: _velocityConnection                                                                                 // 83
    });                                                                                                               // 84
    var cursor = _velocityTestFiles.find({targetFramework: FRAMEWORK_NAME});                                          // 85
                                                                                                                      // 86
    _velocityConnection.onReconnect = function () {                                                                   // 87
      DEBUG && console.log('[xolvio:cucumber] Connected to hub.');                                                    // 88
      var debouncedRun = _.debounce(Meteor.bindEnvironment(_findAndRun), 600);                                        // 89
      cursor.observe({                                                                                                // 90
        added: debouncedRun,                                                                                          // 91
        removed: debouncedRun,                                                                                        // 92
        changed: debouncedRun                                                                                         // 93
      });                                                                                                             // 94
                                                                                                                      // 95
      process.on('SIGUSR2', Meteor.bindEnvironment(function () {                                                      // 96
        DEBUG && console.log('[xolvio:cucumber] SIGUSR2 signal seen');                                                // 97
        debouncedRun.apply(this, arguments);                                                                          // 98
      }));                                                                                                            // 99
      process.on('message', Meteor.bindEnvironment(function (message) {                                               // 100
        DEBUG && console.log('[xolvio:cucumber] Process message seen', message);                                      // 101
        if (message.refresh && message.refresh === 'client') {                                                        // 102
          debouncedRun();                                                                                             // 103
        }                                                                                                             // 104
      }));                                                                                                            // 105
                                                                                                                      // 106
    };                                                                                                                // 107
  }                                                                                                                   // 108
                                                                                                                      // 109
  function _run (feature, cb) {                                                                                       // 110
    if (feature) {                                                                                                    // 111
      _runParallelExecutionMode(feature, cb);                                                                         // 112
    } else {                                                                                                          // 113
      _runSingleExecutionMode();                                                                                      // 114
    }                                                                                                                 // 115
  }                                                                                                                   // 116
                                                                                                                      // 117
  function _runSingleExecutionMode (runAll) {                                                                         // 118
    console.log('[xolvio:cucumber] Cucumber is running'.yellow);                                                      // 119
    _velocityConnection.call('velocity/reports/reset', {framework: FRAMEWORK_NAME});                                  // 120
                                                                                                                      // 121
    try {                                                                                                             // 122
      HTTP.get('http://localhost:' + _getServerPort() + '/interrupt');                                                // 123
                                                                                                                      // 124
                                                                                                                      // 125
      var endPoint = runAll ? '/runAll' : '/run';                                                                     // 126
                                                                                                                      // 127
      var response = HTTP.get('http://localhost:' + _getServerPort() + endPoint);                                     // 128
                                                                                                                      // 129
      var results = response.data;                                                                                    // 130
                                                                                                                      // 131
      if (!results) {                                                                                                 // 132
        console.error('[xolvio:cucumber] Bad response from Chimp server.'.red);                                       // 133
        console.error(response);                                                                                      // 134
        _finishWithError();                                                                                           // 135
        return;                                                                                                       // 136
      }                                                                                                               // 137
                                                                                                                      // 138
      DEBUG && console.log('[xolvio:cucumber] Results from Chimp:');                                                  // 139
      DEBUG && console.log(results);                                                                                  // 140
                                                                                                                      // 141
      if (results.message) {                                                                                          // 142
                                                                                                                      // 143
                                                                                                                      // 144
        var split = results.message.split('\n');                                                                      // 145
                                                                                                                      // 146
        _velocityConnection.call('velocity/reports/submit', {                                                         // 147
                                                                                                                      // 148
          name: results.name,                                                                                         // 149
          framework: FRAMEWORK_NAME,                                                                                  // 150
          result: 'failed',                                                                                           // 151
          failureMessage: split[0],                                                                                   // 152
          failureType: results.message,                                                                               // 153
          failureStackTrace: results.stack || split[1] || ''                                                          // 154
        });                                                                                                           // 155
                                                                                                                      // 156
      }                                                                                                               // 157
                                                                                                                      // 158
      if (results.length === 0) {                                                                                     // 159
        console.log('[xolvio:cucumber] No features found. Be sure to annotate scenarios'.yellow,                      // 160
          'you want to run in watch mode with the'.yellow, WATCH_TAG.cyan, 'tag'.yellow);                             // 161
        _velocityConnection.call('velocity/reports/submit', {                                                         // 162
          name: WATCH_TAG + ' not detected',                                                                          // 163
          framework: FRAMEWORK_NAME,                                                                                  // 164
          ancestors: ['No specs were run'],                                                                           // 165
          result: 'passed'                                                                                            // 166
        });                                                                                                           // 167
      }                                                                                                               // 168
      _processFeatures(results);                                                                                      // 169
    } catch (e) {                                                                                                     // 170
      console.error('[xolvio:cucumber] Bad response from Chimp server.'.red, e);                                      // 171
      _finishWithError();                                                                                             // 172
      return;                                                                                                         // 173
    }                                                                                                                 // 174
    _velocityConnection.call('velocity/reports/completed', {framework: FRAMEWORK_NAME});                              // 175
  }                                                                                                                   // 176
                                                                                                                      // 177
  function _runParallelExecutionMode (feature, cb) {                                                                  // 178
    var _error = false;                                                                                               // 179
                                                                                                                      // 180
    console.log('[xolvio:cucumber] Mirror with pid', process.pid, 'is working on', feature.absolutePath, " port ", _getServerPort());
                                                                                                                      // 182
    try {                                                                                                             // 183
      var response = HTTP.get('http://localhost:' + _getServerPort() + '/run/' + feature.absolutePath);               // 184
      var results = JSON.parse(response.content);                                                                     // 185
      _processFeatures(results);                                                                                      // 186
    }                                                                                                                 // 187
    catch (e) {                                                                                                       // 188
      console.error('[xolvio:cucumber] Bad response from Chimp server.'.red, 'port: '.red, _getServerPort());         // 189
      _error = true;                                                                                                  // 190
    }                                                                                                                 // 191
                                                                                                                      // 192
    if (_error) {                                                                                                     // 193
      if (feature.brokenPreviously) {                                                                                 // 194
        _finishWithError();                                                                                           // 195
      } else {                                                                                                        // 196
        _velocityConnection.call('velocity/featureTestFailed', {featureId: feature._id});                             // 197
      }                                                                                                               // 198
    }                                                                                                                 // 199
    else {                                                                                                            // 200
      _velocityConnection.call('velocity/featureTestDone', {featureId: feature._id});                                 // 201
    }                                                                                                                 // 202
                                                                                                                      // 203
    _runningParallelTest = false;                                                                                     // 204
    cb && cb();                                                                                                       // 205
  }                                                                                                                   // 206
                                                                                                                      // 207
  function _finishWithError (report) {                                                                                // 208
    // attempt to kill any current runs and tell velocity we failed                                                   // 209
                                                                                                                      // 210
    try {                                                                                                             // 211
      HTTP.get('http://localhost:' + _getServerPort() + '/interrupt');                                                // 212
    } catch (e) {                                                                                                     // 213
    }                                                                                                                 // 214
                                                                                                                      // 215
    report = report || {                                                                                              // 216
        name: 'An error occurred. Check the logs for more information.',                                              // 217
        framework: FRAMEWORK_NAME,                                                                                    // 218
        result: 'failed'                                                                                              // 219
      };                                                                                                              // 220
    _velocityConnection.call('velocity/reports/submit', report);                                                      // 221
    _velocityConnection.call('velocity/reports/completed', {framework: FRAMEWORK_NAME});                              // 222
  }                                                                                                                   // 223
                                                                                                                      // 224
  function _startChimp () {                                                                                           // 225
                                                                                                                      // 226
    var cwd = path.resolve(process.env.VELOCITY_MAIN_APP_PATH, 'tests', FRAMEWORK_NAME);                              // 227
                                                                                                                      // 228
    if (!fs.existsSync(cwd)) {                                                                                        // 229
      console.log('[xolvio:cucumber]'.yellow, cwd.yellow, 'does not exist. You need to '.yellow +                     // 230
        'create this directory and add some features to run Cucumber.'.yellow);                                       // 231
      return;                                                                                                         // 232
    }                                                                                                                 // 233
                                                                                                                      // 234
    var startChimpOnPort = function (err, port) {                                                                     // 235
      _chimpServerPort = port;                                                                                        // 236
                                                                                                                      // 237
      DEBUG && console.log('[xolvio:cucumber] Starting Chimp');                                                       // 238
      _chimpProc = new sanjo.LongRunningChildProcess('Chimp_' + _getServerPort());                                    // 239
      if (_chimpProc.isRunning()) {                                                                                   // 240
        DEBUG && console.log('[xolvio:cucumber] Chimp is already running in server mode. Initting');                  // 241
        _init();                                                                                                      // 242
        return;                                                                                                       // 243
      }                                                                                                               // 244
                                                                                                                      // 245
      var args = _getArgs();                                                                                          // 246
                                                                                                                      // 247
      fs.chmodSync(BINARY, parseInt('555', 8));                                                                       // 248
      var nodePath = process.execPath;                                                                                // 249
      var nodeDir = path.dirname(nodePath);                                                                           // 250
      var env = _.clone(process.env);                                                                                 // 251
      // Expose the Meteor node binary path for the script that is run                                                // 252
      env.PATH = nodeDir + ':' + env.PATH;                                                                            // 253
      var spawnOptions = {                                                                                            // 254
        cwd: cwd,                                                                                                     // 255
        stdio: 'inherit',                                                                                             // 256
        env: env                                                                                                      // 257
      };                                                                                                              // 258
                                                                                                                      // 259
      args.push('--server');                                                                                          // 260
                                                                                                                      // 261
      args.push('--serverPort=' + _getServerPort());                                                                  // 262
                                                                                                                      // 263
      DEBUG && console.log('[xolvio:cucumber] Starting Chimp with', BINARY, args, spawnOptions);                      // 264
                                                                                                                      // 265
      _chimpProc.spawn({                                                                                              // 266
        command: nodePath,                                                                                            // 267
        args: args,                                                                                                   // 268
        options: spawnOptions                                                                                         // 269
      });                                                                                                             // 270
                                                                                                                      // 271
      DEBUG && console.log('[xolvio:cucumber] Chimp process forked with pid', _chimpProc.getPid());                   // 272
                                                                                                                      // 273
    };                                                                                                                // 274
                                                                                                                      // 275
    if (!!process.env.CUCUMBER_NODES) {                                                                               // 276
      freeport(Meteor.bindEnvironment(startChimpOnPort))                                                              // 277
    } else {                                                                                                          // 278
      var serverPort = process.env.CUCUMBER_SERVER_PORT ? process.env.CUCUMBER_SERVER_PORT : 8866;                    // 279
      startChimpOnPort(null, serverPort);                                                                             // 280
    }                                                                                                                 // 281
                                                                                                                      // 282
  }                                                                                                                   // 283
                                                                                                                      // 284
  function _getServerPort () {                                                                                        // 285
    return _chimpServerPort;                                                                                          // 286
  }                                                                                                                   // 287
                                                                                                                      // 288
  function _getScreenshotsDir () {                                                                                    // 289
    var _screenshotsDir = process.env.CUCUMBER_SCREENSHOTS_DIR ||                                                     // 290
      path.resolve(process.env.VELOCITY_MAIN_APP_PATH, 'tests', FRAMEWORK_NAME, '.screenshots');                      // 291
    DEBUG && console.log('[xolvio:cucumber] Screenshots dir is', _screenshotsDir);                                    // 292
    return path.resolve(_screenshotsDir);                                                                             // 293
  }                                                                                                                   // 294
                                                                                                                      // 295
  function _getArgs () {                                                                                              // 296
                                                                                                                      // 297
    var args = [];                                                                                                    // 298
                                                                                                                      // 299
    if (process.env.CHIMP_NODE_OPTIONS) {                                                                             // 300
      args.push(process.env.CHIMP_NODE_OPTIONS);                                                                      // 301
    } else if (process.env.CHIMP_DEBUG) {                                                                             // 302
      args.push('--debug-brk');                                                                                       // 303
    }                                                                                                                 // 304
                                                                                                                      // 305
    args.push(BINARY);                                                                                                // 306
                                                                                                                      // 307
    args.push('-r');                                                                                                  // 308
    args.push(path.join(process.env.VELOCITY_MAIN_APP_PATH, 'tests', 'cucumber', 'features'));                        // 309
    args.push('--snippets');                                                                                          // 310
    args.push('--ddp=' + process.env.ROOT_URL);                                                                       // 311
    args.push('--log=error');                                                                                         // 312
                                                                                                                      // 313
                                                                                                                      // 314
    // TODO only if local / xolvio:reporter is detected                                                               // 315
    //args.push('--simianResultEndPoint=' + process.env.PARENT_URL.substring('http://'.length) + 'xolvio/reporter/results');
    //args.push('--simianAccessToken=' + 'local');                                                                    // 317
                                                                                                                      // 318
    if (DEBUG || process.env.DEBUG) {                                                                                 // 319
      args.push('--debug');                                                                                           // 320
    }                                                                                                                 // 321
                                                                                                                      // 322
    if (process.env.CHIMP_OPTIONS) {                                                                                  // 323
      var chimpOptions = process.env.CHIMP_OPTIONS.split(' ');                                                        // 324
      while (chimpOptions.length != 0) {                                                                              // 325
        args.push(chimpOptions.pop());                                                                                // 326
      }                                                                                                               // 327
      return args;                                                                                                    // 328
    }                                                                                                                 // 329
                                                                                                                      // 330
    if (process.env.VELOCITY_CI) {                                                                                    // 331
      args.push('--screenshotsOnError=true');                                                                         // 332
    }                                                                                                                 // 333
                                                                                                                      // 334
    args.push('--screenshotsPath=' + _getScreenshotsDir());                                                           // 335
                                                                                                                      // 336
    if (process.env.CUCUMBER_COFFEE_SNIPPETS) {                                                                       // 337
      args.push('--coffee');                                                                                          // 338
    }                                                                                                                 // 339
                                                                                                                      // 340
    if (process.env.CUCUMBER_TAGS) {                                                                                  // 341
      args.push('--tags=' + process.env.CUCUMBER_TAGS);                                                               // 342
    } else if (!process.env.VELOCITY_CI) {                                                                            // 343
      args.push('--tags=' + WATCH_TAG);                                                                               // 344
    } else {                                                                                                          // 345
      args.push('--tags=~@ignore');                                                                                   // 346
    }                                                                                                                 // 347
                                                                                                                      // 348
    if (process.env.CUCUMBER_FORMAT) {                                                                                // 349
      args.push('--format=' + process.env.CUCUMBER_FORMAT);                                                           // 350
    }                                                                                                                 // 351
                                                                                                                      // 352
    if (process.env.SIMIAN_ACCESS_TOKEN) {                                                                            // 353
      args.push('--simianAccessToken=' + process.env.SIMIAN_ACCESS_TOKEN);                                            // 354
    }                                                                                                                 // 355
                                                                                                                      // 356
    if (process.env.CUCUMBER_JSON_OUTPUT) {                                                                           // 357
      args.push('--jsonOutput=' + process.env.CUCUMBER_JSON_OUTPUT);                                                  // 358
    }                                                                                                                 // 359
                                                                                                                      // 360
    if (process.env.WD_TIMEOUT_ASYNC_SCRIPT) {                                                                        // 361
      args.push('--timeoutsAsyncScript=' + process.env.WD_TIMEOUT_ASYNC_SCRIPT);                                      // 362
    }                                                                                                                 // 363
                                                                                                                      // 364
    if (process.env.SELENIUM_BROWSER) {                                                                               // 365
      args.push('--browser=' + process.env.SELENIUM_BROWSER);                                                         // 366
    }                                                                                                                 // 367
                                                                                                                      // 368
    if (process.env.HUB_HOST) {                                                                                       // 369
      args.push('--host=' + process.env.HUB_HOST);                                                                    // 370
    }                                                                                                                 // 371
                                                                                                                      // 372
    if (process.env.HUB_PORT) {                                                                                       // 373
      args.push('--port=' + process.env.HUB_PORT);                                                                    // 374
    }                                                                                                                 // 375
                                                                                                                      // 376
    if (process.env.HUB_USER) {                                                                                       // 377
      args.push('--user=' + process.env.HUB_USER);                                                                    // 378
    }                                                                                                                 // 379
                                                                                                                      // 380
    if (process.env.HUB_KEY) {                                                                                        // 381
      args.push('--key=' + process.env.HUB_KEY);                                                                      // 382
    }                                                                                                                 // 383
                                                                                                                      // 384
    if (process.env.HUB_PLATFORM) {                                                                                   // 385
      args.push('--platform=' + process.env.HUB_PLATFORM);                                                            // 386
    }                                                                                                                 // 387
                                                                                                                      // 388
    if (process.env.HUB_VERSION) {                                                                                    // 389
      args.push('--version=' + process.env.HUB_VERSION);                                                              // 390
    }                                                                                                                 // 391
                                                                                                                      // 392
    if (process.env.CHIMP_OFFLINE) {                                                                                  // 393
      args.push('--offline');                                                                                         // 394
    }                                                                                                                 // 395
                                                                                                                      // 396
    if (process.env.DEBUG_CUCUMBER) {                                                                                 // 397
      args.push('--debugCucumber=' + process.env.DEBUG_CUCUMBER);                                                     // 398
    }                                                                                                                 // 399
                                                                                                                      // 400
    if (process.env.DEBUG_BRK_CUCUMBER) {                                                                             // 401
      args.push('--debugBrkCucumber=' + process.env.DEBUG_BRK_CUCUMBER);                                              // 402
    }                                                                                                                 // 403
                                                                                                                      // 404
    return args;                                                                                                      // 405
                                                                                                                      // 406
  }                                                                                                                   // 407
                                                                                                                      // 408
  function _processFeatures (features) {                                                                              // 409
    _.each(features, function (feature) {                                                                             // 410
      _processFeature(feature);                                                                                       // 411
    });                                                                                                               // 412
  }                                                                                                                   // 413
                                                                                                                      // 414
  function _processFeature (feature) {                                                                                // 415
    _.each(feature.elements, function (element) {                                                                     // 416
      _processFeatureElements(element, feature);                                                                      // 417
    });                                                                                                               // 418
  }                                                                                                                   // 419
                                                                                                                      // 420
  function _processFeatureElements (element, feature) {                                                               // 421
    _.each(element.steps, function (step) {                                                                           // 422
      _processStep(element, step, feature);                                                                           // 423
    });                                                                                                               // 424
  }                                                                                                                   // 425
                                                                                                                      // 426
  function _processStep (element, step, feature) {                                                                    // 427
                                                                                                                      // 428
    if (element.type === 'background') {                                                                              // 429
      return;                                                                                                         // 430
    }                                                                                                                 // 431
                                                                                                                      // 432
    var report = {                                                                                                    // 433
      id: element.id + step.keyword + step.name,                                                                      // 434
      framework: FRAMEWORK_NAME,                                                                                      // 435
      name: step.keyword + step.name,                                                                                 // 436
      result: step.result.status,                                                                                     // 437
      ancestors: [element.name, feature.name]                                                                         // 438
    };                                                                                                                // 439
    if (step.result.duration) {                                                                                       // 440
      report.duration = Math.round(step.result.duration / 1000000);                                                   // 441
    }                                                                                                                 // 442
    if (step.result.error_message) {                                                                                  // 443
                                                                                                                      // 444
      if (step.result.error_message.name) {                                                                           // 445
        report.failureType = step.result.error_message.name;                                                          // 446
        report.failureStackTrace = _conditionError(step.result.error_message.message);                                // 447
      } else {                                                                                                        // 448
        report.failureStackTrace = _conditionError(step.result.error_message);                                        // 449
      }                                                                                                               // 450
    }                                                                                                                 // 451
                                                                                                                      // 452
    if (step.result.status === 'undefined') {                                                                         // 453
      report.result = 'failed';                                                                                       // 454
      report.failureStackTrace = 'This step has not been defined';                                                    // 455
    }                                                                                                                 // 456
                                                                                                                      // 457
    if (step.result.status === 'skipped') {                                                                           // 458
      report.result = 'failed';                                                                                       // 459
      report.failureStackTrace = 'This step was skipped';                                                             // 460
    }                                                                                                                 // 461
                                                                                                                      // 462
    if (step.result.status === 'pending') {                                                                           // 463
      report.result = 'failed';                                                                                       // 464
      report.failureStackTrace = 'This step is pending';                                                              // 465
    }                                                                                                                 // 466
                                                                                                                      // 467
    // skip before/after if they have no errors                                                                       // 468
    if (!report.failureStackTrace && (step.keyword.trim() === 'Before' || step.keyword.trim() === 'After')) {         // 469
      return;                                                                                                         // 470
    }                                                                                                                 // 471
                                                                                                                      // 472
    _velocityConnection.call('velocity/reports/submit', report);                                                      // 473
                                                                                                                      // 474
  }                                                                                                                   // 475
                                                                                                                      // 476
  function _conditionError (errorMessage) {                                                                           // 477
                                                                                                                      // 478
    if (DEBUG) {                                                                                                      // 479
      console.error(errorMessage.red);                                                                                // 480
      return errorMessage;                                                                                            // 481
    }                                                                                                                 // 482
                                                                                                                      // 483
    var msg        = '',                                                                                              // 484
        insertDots = false,                                                                                           // 485
        DOTS       = '  ...\n';                                                                                       // 486
                                                                                                                      // 487
    try {                                                                                                             // 488
                                                                                                                      // 489
      if (errorMessage.indexOf('Timed out waiting for asyncrhonous') !== -1) {                                        // 490
        return errorMessage.substring(errorMessage.lastIndexOf('->') + 3, errorMessage.length).trim()                 // 491
          + ' timed out';                                                                                             // 492
      }                                                                                                               // 493
                                                                                                                      // 494
      _.each(errorMessage.split('\n'), function (line, index) {                                                       // 495
        line = line.trim();                                                                                           // 496
        if (index === 0) {                                                                                            // 497
          msg += line;                                                                                                // 498
          return;                                                                                                     // 499
        }                                                                                                             // 500
        if (line.indexOf('/tests/cucumber/') !== -1) {                                                                // 501
          if (insertDots) {                                                                                           // 502
            msg += DOTS;                                                                                              // 503
            insertDots = false;                                                                                       // 504
          }                                                                                                           // 505
          msg += ((index !== 0 ? '  ' : '') +                                                                         // 506
          line.replace(process.env.VELOCITY_MAIN_APP_PATH, '') + '\n');                                               // 507
        } else {                                                                                                      // 508
          insertDots = true;                                                                                          // 509
        }                                                                                                             // 510
      });                                                                                                             // 511
      if (insertDots) {                                                                                               // 512
        msg += DOTS;                                                                                                  // 513
      }                                                                                                               // 514
                                                                                                                      // 515
      // TODO deal with this error for users                                                                          // 516
      // Cannot call method 'convertScenarioOutlinesToScenarios' of undefined                                         // 517
                                                                                                                      // 518
      msg = msg.substring(0, msg.length);                                                                             // 519
      //console.log(msg.magenta);                                                                                     // 520
    } catch (e) {                                                                                                     // 521
      msg = errorMessage;                                                                                             // 522
      console.error(errorMessage);                                                                                    // 523
    }                                                                                                                 // 524
                                                                                                                      // 525
    return msg;                                                                                                       // 526
  }                                                                                                                   // 527
                                                                                                                      // 528
})();                                                                                                                 // 529
                                                                                                                      // 530
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['xolvio:cucumber'] = {};

})();

//# sourceMappingURL=xolvio_cucumber.js.map
